

_NASA_EXOPLANET_ARCHIVE_COMPOSITE_MAPPER = {
    # Basic names
    'pl_name': 'Planet.Name',
    'hostname': 'Star.Name',
    'pl_letter': 'Planet.Letter',
    'hd_name': 'Star.Catalogue.HD',
    'hip_name': 'Star.Catalogue.HIP',
    'tic_id': 'Star.Catalogue.TIC',
    'gaia_id': 'Star.Catalogue.GAIA',
    # System composition
    'sy_snum': 'System.NumberOfStars',
    'sy_pnum': 'System.NumberOfPlanets',
    'sy_mnum': 'System.NumberOfMoons',
    'cb_flag': 'Flag.BinarySystem',
    # Planet discovery
    'discoverymethod': 'Discovery.Method',
    'disc_year': 'Discovery.Year',
    'disc_refname': 'Discovery.Reference',
    'disc_pubdate': 'Discovery.PublicationDate',
    'disc_locale': 'Discovery.Locale',
    'disc_facility': 'Discovery.Facility',
    'disc_telescope': 'Discovery.Telescope',
    'disc_instrument': 'Discovery.Instrument',
    # Detections
    'rv_flag': 'Flag.RadialVelocity',
    'pul_flag': 'Flag.PulsarTimingVariations',
    'ptv_flag': 'Flag.PulsationTimingVariations',
    'tran_flag': 'Flag.Transit',
    'ast_flag': 'Flag.AstrometricVariations',
    'obm_flag': 'Flag.OrbitalBrightnessModulations',
    'micro_flag': 'Flag.Microlensing',
    'etv_flag': 'Flag.EclipseTimingVariations',
    'ima_flag': 'Flag.Imaging',
    'dkin_flag': 'Flag.DiskKinematics',
    'ttv_flag': 'Flag.TransitTimingVariations',
    
    # Planet parameters
    'pl_refname': 'Planet.Reference',
    
    'pl_orbper': 'Planet.Period',
    'pl_orbpererr1': 'Planet.Period.Error.Upper',
    'pl_orbpererr2': 'Planet.Period.Error.Lower',
    'pl_orbper_reflink': 'Planet.Period.Reference',

    'pl_orbsmax': 'Planet.SemiMajorAxis',
    'pl_orbsmaxerr1': 'Planet.SemiMajorAxis.Error.Upper',
    'pl_orbsmaxerr2': 'Planet.SemiMajorAxis.Error.Lower',
    'pl_orbsmax_reflink': 'Planet.SemiMajorAxis.Reference',

    'pl_angsep': 'Planet.AngularSeparation',
    'pl_angseperr1': 'Planet.AngularSeparation.Error.Upper',
    'pl_angseperr2': 'Planet.AngularSeparation.Error.Lower',
    'pl_angsep_reflink': 'Planet.AngularSeparation.Reference',

    'pl_rade': 'Planet.RadiusEarth',
    'pl_radeerr1': 'Planet.RadiusEarth.Error.Upper',
    'pl_radeerr2': 'Planet.RadiusEarth.Error.Lower',
    'pl_rade_reflink': 'Planet.RadiusEarth.Reference',

    'pl_radj': 'Planet.RadiusJupiter',
    'pl_radjerr1': 'Planet.RadiusJupiter.Error.Upper',
    'pl_radjerr2': 'Planet.RadiusJupiter.Error.Lower',
    'pl_radj_reflink': 'Planet.RadiusJupiter.Reference',

    'pl_masse': 'Planet.MassEarth',
    'pl_masseerr1': 'Planet.MassEarth.Error.Upper',
    'pl_masseerr2': 'Planet.MassEarth.Error.Lower',
    'pl_masse_reflink': 'Planet.MassEarth.Reference',

    'pl_massj': 'Planet.MassJupiter',
    'pl_massjerr1': 'Planet.MassJupiter.Error.Upper',
    'pl_massjerr2': 'Planet.MassJupiter.Error.Lower',
    'pl_massj_reflink': 'Planet.MassJupiter.Reference',
    
    'pl_msinie': 'Planet.MinimumMassEarth',
    'pl_msinieerr1': 'Planet.MinimumMassEarth.Error.Upper',
    'pl_msinieerr2': 'Planet.MinimumMassEarth.Error.Lower',
    'pl_msinie_reflink': 'Planet.MinimumMassEarth.Reference',

    'pl_msinij': 'Planet.MinimumMassJupiter',
    'pl_msinijerr1': 'Planet.MinimumMassJupiter.Error.Upper',
    'pl_msinijerr2': 'Planet.MinimumMassJupiter.Error.Lower',
    'pl_msinij_reflink': 'Planet.MinimumMassJupiter.Reference',
    
    'pl_bmasse': 'Planet.BestMassEstimateEarth',
    'pl_bmasseerr1': 'Planet.BestMassEstimateEarth.Error.Upper',
    'pl_bmasseerr2': 'Planet.BestMassEstimateEarth.Error.Lower',
    'pl_bmasse_reflink': 'Planet.BestMassEstimateEarth.Reference',

    'pl_bmassj': 'Planet.BestMassEstimateJupiter',
    'pl_bmassjerr1': 'Planet.BestMassEstimateJupiter.Error.Upper',
    'pl_bmassjerr2': 'Planet.BestMassEstimateJupiter.Error.Lower',
    'pl_bmassj_reflink': 'Planet.BestMassEstimateJupiter.Reference',
    
    'pl_dens': 'Planet.Density',
    'pl_denserr1': 'Planet.Density.Error.Upper',
    'pl_denserr2': 'Planet.Density.Error.Lower',
    'pl_dens_reflink': 'Planet.Density.Reference',
    
    'pl_orbeccen': 'Planet.Eccentricity',
    'pl_orbeccenerr1': 'Planet.Eccentricity.Error.Upper',
    'pl_orbeccenerr2': 'Planet.Eccentricity.Error.Lower',
    'pl_orbeccen_reflink': 'Planet.Eccentricity.Reference',
    
    'pl_insol': 'Planet.InsolationFlux',
    'pl_insolerr1': 'Planet.InsolationFlux.Error.Upper',
    'pl_insolerr2': 'Planet.InsolationFlux.Error.Lower',
    'pl_insol_reflink': 'Planet.InsolationFlux.Reference',
    
    'pl_eqt': 'Planet.EquilibriumTemperature',
    'pl_eqterr1': 'Planet.EquilibriumTemperature.Error.Upper',
    'pl_eqterr2': 'Planet.EquilibriumTemperature.Error.Lower',
    'pl_eqt_reflink': 'Planet.EquilibriumTemperature.Reference',

    'pl_orbincl': 'Planet.Inclination',
    'pl_orbinclerr1': 'Planet.Inclination.Error.Upper',
    'pl_orbinclerr2': 'Planet.Inclination.Error.Lower',
    'pl_orbincl_reflink': 'Planet.Inclination.Reference',

    'pl_tranmid': 'Planet.TransitMidpoint',
    'pl_tranmiderr1': 'Planet.TransitMidpoint.Error.Upper',
    'pl_tranmiderr2': 'Planet.TransitMidpoint.Error.Lower',
    'pl_tranmid_reflink': 'Planet.TransitMidpoint.Reference',

    'pl_imppar': 'Planet.ImpactParameter',
    'pl_impparerr1': 'Planet.ImpactParameter.Error.Upper',
    'pl_impparerr2': 'Planet.ImpactParameter.Error.Lower',
    'pl_imppar_reflink': 'Planet.ImpactParameter.Reference',

    'pl_trandep': 'Planet.TransitDepth',
    'pl_trandeperr1': 'Planet.TransitDepth.Error.Upper',
    'pl_trandeperr2': 'Planet.TransitDepth.Error.Lower',
    'pl_trandep_reflink': 'Planet.TransitDepth.Reference',

    'pl_trandur': 'Planet.TransitDuration',
    'pl_trandurerr1': 'Planet.TransitDuration.Error.Upper',
    'pl_trandurerr2': 'Planet.TransitDuration.Error.Lower',
    'pl_trandur_reflink': 'Planet.TransitDuration.Reference',

    'pl_ratdor': 'Planet.RatioSemiMajorAxisToStellarRadius',
    'pl_ratdorerr1': 'Planet.RatioSemiMajorAxisToStellarRadius.Error.Upper',
    'pl_ratdorerr2': 'Planet.RatioSemiMajorAxisToStellarRadius.Error.Lower',
    'pl_ratdor_reflink': 'Planet.RatioSemiMajorAxisToStellarRadius.Reference',

    'pl_ratror': 'Planet.RatioPlanetRadiusToStellarRadius',
    'pl_ratrorerr1': 'Planet.RatioPlanetRadiusToStellarRadius.Error.Upper',
    'pl_ratrorerr2': 'Planet.RatioPlanetRadiusToStellarRadius.Error.Lower',
    'pl_ratror_reflink': 'Planet.RatioPlanetRadiusToStellarRadius.Reference',

    'pl_occdep': 'Planet.OccultationDepth',
    'pl_occdeperr1': 'Planet.OccultationDepth.Error.Upper',
    'pl_occdeperr2': 'Planet.OccultationDepth.Error.Lower',
    'pl_occdep_reflink': 'Planet.OccultationDepth.Reference',

    'pl_orbtper': 'Planet.EpochOfPeriastron',
    'pl_orbtpererr1': 'Planet.EpochOfPeriastron.Error.Upper',
    'pl_orbtpererr2': 'Planet.EpochOfPeriastron.Error.Lower',
    'pl_orbtper_reflink': 'Planet.EpochOfPeriastron.Reference',

    'pl_orblper': 'Planet.ArgumentOfPeriastron',
    'pl_orblpererr1': 'Planet.ArgumentOfPeriastron.Error.Upper',
    'pl_orblpererr2': 'Planet.ArgumentOfPeriastron.Error.Lower',
    'pl_orblper_reflink': 'Planet.ArgumentOfPeriastron.Reference',

    'pl_rvamp': 'Planet.RadialVelocityAmplitude',
    'pl_rvamperr1': 'Planet.RadialVelocityAmplitude.Error.Upper',
    'pl_rvamperr2': 'Planet.RadialVelocityAmplitude.Error.Lower',
    'pl_rvamp_reflink': 'Planet.RadialVelocityAmplitude.Reference',

    'pl_projobliq': 'Planet.ProjectedObliquity',
    'pl_projobliqerr1': 'Planet.ProjectedObliquity.Error.Upper',
    'pl_projobliqerr2': 'Planet.ProjectedObliquity.Error.Lower',
    'pl_projobliq_reflink': 'Planet.ProjectedObliquity.Reference',

    'pl_trueobliq': 'Planet.TrueObliquity',
    'pl_trueobliqerr1': 'Planet.TrueObliquity.Error.Upper',
    'pl_trueobliqerr2': 'Planet.TrueObliquity.Error.Lower',
    'pl_trueobliq_reflink': 'Planet.TrueObliquity.Reference',

    # Star parameters
    'st_refname': 'Star.Reference',
    'st_spectype': 'Star.Type',
    'st_spectype': 'Star.Type.Reference',
    
    'st_teff': 'Star.EffectiveTemperature',
    'st_tefferr1': 'Star.EffectiveTemperature.Error.Upper',
    'st_tefferr2': 'Star.EffectiveTemperature.Error.Lower',
    'st_teff_reflink': 'Star.EffectiveTemperature.Reference',
    
    'st_rad': 'Star.Radius',
    'st_raderr1': 'Star.Radius.Error.Upper',
    'st_raderr2': 'Star.Radius.Error.Lower',
    'st_rad_reflink': 'Star.Radius.Reference',
    
    'st_mass': 'Star.Mass',
    'st_masserr1': 'Star.Mass.Error.Upper',
    'st_masserr2': 'Star.Mass.Error.Lower',
    'st_mass_reflink': 'Star.Mass.Reference',

    'st_met': 'Star.Metallicity',
    'st_meterr1': 'Star.Metallicity.Error.Upper',
    'st_meterr2': 'Star.Metallicity.Error.Lower',
    'st_met_reflink': 'Star.Metallicity.Reference',

    'st_metratio': 'Star.MetallicityRatio',
    'st_metratioerr1': 'Star.MetallicityRatio.Error.Upper',
    'st_metratioerr2': 'Star.MetallicityRatio.Error.Lower',
    'st_metratio_reflink': 'Star.MetallicityRatio.Reference',

    'st_lum': 'Star.Luminosity',
    'st_lumerr1': 'Star.Luminosity.Error.Upper',
    'st_lumerr2': 'Star.Luminosity.Error.Lower',
    'st_lum_reflink': 'Star.Luminosity.Reference',
    
    'st_logg': 'Star.Logg',
    'st_loggerr1': 'Star.Logg.Error.Upper',
    'st_loggerr2': 'Star.Logg.Error.Lower',
    'st_logg_reflink': 'Star.Logg.Reference',

    'st_age': 'Star.Age',
    'st_ageerr1': 'Star.Age.Error.Upper',
    'st_ageerr2': 'Star.Age.Error.Lower',
    'st_age_reflink': 'Star.Age.Reference',

    'st_dens': 'Star.Density',
    'st_denserr1': 'Star.Density.Error.Upper',
    'st_denserr2': 'Star.Density.Error.Lower',
    'st_dens_reflink': 'Star.Density.Reference',

    'st_vsin': 'Star.RotationalVelocity',
    'st_vsinerr1': 'Star.RotationalVelocity.Error.Upper',
    'st_vsinerr2': 'Star.RotationalVelocity.Error.Lower',
    'st_vsin_reflink': 'Star.RotationalVelocity.Reference',

    'st_rotp': 'Star.RotationalPeriod',
    'st_rotperr1': 'Star.RotationalPeriod.Error.Upper',
    'st_rotperr2': 'Star.RotationalPeriod.Error.Lower',
    'st_rotp_reflink': 'Star.RotationalPeriod.Reference',

    # Technically under star, moved to System
    'st_radv': 'System.Velocity',
    'st_radverr1': 'System.Velocity.Error.Upper',
    'st_radverr2': 'System.Velocity.Error.Lower',
    'st_radv_reflink': 'System.Velocity.Reference',

    # System parameters
    'sy_refname': 'System.Reference',
    
    'sy_pm': 'System.TotalProperMotion',
    'sy_pmerr1': 'System.TotalProperMotion.Error.Upper',
    'sy_pmerr2': 'System.TotalProperMotion.Error.Lower',
    'sy_pm_reflink': 'System.TotalProperMotion.Reference',

    'sy_pmra': 'System.ProperMotionRightAscension',
    'sy_pmraerr1': 'System.ProperMotionRightAscension.Error.Upper',
    'sy_pmraerr2': 'System.ProperMotionRightAscension.Error.Lower',
    'sy_pmra_reflink': 'System.ProperMotionRightAscension.Reference',

    'sy_pmdec': 'System.ProperMotionDeclination',
    'sy_pmdecerr1': 'System.ProperMotionDeclination.Error.Upper',
    'sy_pmdecerr2': 'System.ProperMotionDeclination.Error.Lower',
    'sy_pmdec_reflink': 'System.ProperMotionDeclination.Reference',

    'sy_dist': 'System.Distance',
    'sy_disterr1': 'System.Distance.Error.Upper',
    'sy_disterr2': 'System.Distance.Error.Lower',
    'sy_dist_reflink': 'System.Distance.Reference',

    'sy_plx': 'System.Parallax',
    'sy_plxerr1': 'System.Parallax.Error.Upper',
    'sy_plxerr2': 'System.Parallax.Error.Lower',
    'sy_plx_reflink': 'System.Parallax.Reference',
    
    # Position keywords
    'rastr': 'Position.RightAscensionString',
    'decst': 'Position.DeclinationString',

    'ra': 'Position.RightAscension',
    'raerr1': 'Position.RightAscension.Error.Upper',
    'raerr2': 'Position.RightAscension.Error.Lower',
    'ra_reflink': 'Position.RightAscension.Reference',

    'dec': 'Position.Declination',
    'decerr1': 'Position.Declination.Error.Upper',
    'decerr2': 'Position.Declination.Error.Lower',
    'dec_reflink': 'Position.Declination.Reference',

    'glat': 'Position.GalacticLatitude',
    'glaterr1': 'Position.GalacticLatitude.Error.Upper',
    'glaterr2': 'Position.GalacticLatitude.Error.Lower',
    'glat_reflink': 'Position.GalacticLatitude.Reference',

    'glon': 'Position.GalacticLongitude',
    'glonerr1': 'Position.GalacticLongitude.Error.Upper',
    'glonerr2': 'Position.GalacticLongitude.Error.Lower',
    'glon_reflink': 'Position.GalacticLongitude.Reference',

    'elat': 'Position.EclipticLatitude',
    'elaterr1': 'Position.EclipticLatitude.Error.Upper',
    'elaterr2': 'Position.EclipticLatitude.Error.Lower',
    'elat_reflink': 'Position.EclipticLatitude.Reference',

    'elon': 'Position.EclipticLongitude',
    'elonerr1': 'Position.EclipticLongitude.Error.Upper',
    'elonerr2': 'Position.EclipticLongitude.Error.Lower',
    'elon_reflink': 'Position.EclipticLongitude.Reference',
    
    # Magnitude information
    'sy_bmag': 'Magnitude.B',
    'sy_bmagerr1': 'Magnitude.B.Error.Upper',
    'sy_bmagerr2': 'Magnitude.B.Error.Lower',
    'sy_bmag_reflink': 'Magnitude.B.Reference',

    'sy_vmag': 'Magnitude.V',
    'sy_vmagerr1': 'Magnitude.V.Error.Upper',
    'sy_vmagerr2': 'Magnitude.V.Error.Lower',
    'sy_vmag_reflink': 'Magnitude.V.Reference',

    'sy_jmag': 'Magnitude.J',
    'sy_jmagerr1': 'Magnitude.J.Error.Upper',
    'sy_jmagerr2': 'Magnitude.J.Error.Lower',
    'sy_jmag_reflink': 'Magnitude.J.Reference',

    'sy_hmag': 'Magnitude.H',
    'sy_hmagerr1': 'Magnitude.H.Error.Upper',
    'sy_hmagerr2': 'Magnitude.H.Error.Lower',
    'sy_hmag_reflink': 'Magnitude.H.Reference',

    'sy_kmag': 'Magnitude.K',
    'sy_kmagerr1': 'Magnitude.K.Error.Upper',
    'sy_kmagerr2': 'Magnitude.K.Error.Lower',
    'sy_kmag_reflink': 'Magnitude.K.Reference',

    'sy_umag': 'Magnitude.u',
    'sy_umagerr1': 'Magnitude.u.Error.Upper',
    'sy_umagerr2': 'Magnitude.u.Error.Lower',
    'sy_umag_reflink': 'Magnitude.u.Reference',

    'sy_gmag': 'Magnitude.g',
    'sy_gmagerr1': 'Magnitude.g.Error.Upper',
    'sy_gmagerr2': 'Magnitude.g.Error.Lower',
    'sy_gmag_reflink': 'Magnitude.g.Reference',

    'sy_rmag': 'Magnitude.r',
    'sy_rmagerr1': 'Magnitude.r.Error.Upper',
    'sy_rmagerr2': 'Magnitude.r.Error.Lower',
    'sy_rmag_reflink': 'Magnitude.r.Reference',

    'sy_imag': 'Magnitude.i',
    'sy_imagerr1': 'Magnitude.i.Error.Upper',
    'sy_imagerr2': 'Magnitude.i.Error.Lower',
    'sy_imag_reflink': 'Magnitude.i.Reference',

    'sy_zmag': 'Magnitude.z',
    'sy_zmagerr1': 'Magnitude.z.Error.Upper',
    'sy_zmagerr2': 'Magnitude.z.Error.Lower',
    'sy_zmag_reflink': 'Magnitude.z.Reference',

    'sy_w1mag': 'Magnitude.W1',
    'sy_w1magerr1': 'Magnitude.W1.Error.Upper',
    'sy_w1magerr2': 'Magnitude.W1.Error.Lower',
    'sy_w1mag_reflink': 'Magnitude.W1.Reference',

    'sy_w2mag': 'Magnitude.W2',
    'sy_w2magerr1': 'Magnitude.W2.Error.Upper',
    'sy_w2magerr2': 'Magnitude.W2.Error.Lower',
    'sy_w2mag_reflink': 'Magnitude.W2.Reference',

    'sy_w3mag': 'Magnitude.W3',
    'sy_w3magerr1': 'Magnitude.W3.Error.Upper',
    'sy_w3magerr2': 'Magnitude.W3.Error.Lower',
    'sy_w3mag_reflink': 'Magnitude.W3.Reference',

    'sy_w4mag': 'Magnitude.W4',
    'sy_w4magerr1': 'Magnitude.W4.Error.Upper',
    'sy_w4magerr2': 'Magnitude.W4.Error.Lower',
    'sy_w4mag_reflink': 'Magnitude.W4.Reference',

    'sy_gaiamag': 'Magnitude.Gaia',
    'sy_gaiamagerr1': 'Magnitude.Gaia.Error.Upper',
    'sy_gaiamagerr2': 'Magnitude.Gaia.Error.Lower',
    'sy_gaiamag_reflink': 'Magnitude.Gaia.Reference',

    'sy_icmag': 'Magnitude.IC',
    'sy_icmagerr1': 'Magnitude.IC.Error.Upper',
    'sy_icmagerr2': 'Magnitude.IC.Error.Lower',
    'sy_icmag_reflink': 'Magnitude.IC.Reference',

    'sy_tmag': 'Magnitude.TESS',
    'sy_tmagerr1': 'Magnitude.TESS.Error.Upper',
    'sy_tmagerr2': 'Magnitude.TESS.Error.Lower',
    'sy_tmag_reflink': 'Magnitude.TESS.Reference',

    'sy_kepmag': 'Magnitude.Kepler',
    'sy_kepmagerr1': 'Magnitude.Kepler.Error.Upper',
    'sy_kepmagerr2': 'Magnitude.Kepler.Error.Lower',
    'sy_kepmag_reflink': 'Magnitude.Kepler.Reference',

}

_NASA_EXOPLANET_ARCHIVE_FULL_MAPPER = {
    # Basic names
    'pl_name': 'Planet.Name',
    'hostname': 'Star.Name',
    'pl_letter': 'Planet.Letter',
    'hd_name': 'Star.Catalogue.HD',
    'hip_name': 'Star.Catalogue.HIP',
    'tic_id': 'Star.Catalogue.TIC',
    'gaia_id': 'Star.Catalogue.GAIA',
    # System composition
    'sy_snum': 'System.NumberOfStars',
    'sy_pnum': 'System.NumberOfPlanets',
    'sy_mnum': 'System.NumberOfMoons',
    'cb_flag': 'Flag.BinarySystem',
    # Planet discovery
    'discoverymethod': 'Discovery.Method',
    'disc_year': 'Discovery.Year',
    'disc_refname': 'Discovery.Reference',
    'disc_pubdate': 'Discovery.PublicationDate',
    'disc_locale': 'Discovery.Locale',
    'disc_facility': 'Discovery.Facility',
    'disc_telescope': 'Discovery.Telescope',
    'disc_instrument': 'Discovery.Instrument',
    # Detections
    'rv_flag': 'Flag.RadialVelocity',
    'pul_flag': 'Flag.PulsarTimingVariations',
    'ptv_flag': 'Flag.PulsationTimingVariations',
    'tran_flag': 'Flag.Transit',
    'ast_flag': 'Flag.AstrometricVariations',
    'obm_flag': 'Flag.OrbitalBrightnessModulations',
    'micro_flag': 'Flag.Microlensing',
    'etv_flag': 'Flag.EclipseTimingVariations',
    'ima_flag': 'Flag.Imaging',
    'dkin_flag': 'Flag.DiskKinematics',
    'ttv_flag': 'Flag.TransitTimingVariations',
    
    # Planet parameters
    'pl_refname': 'Planet.Reference',
    
    'pl_orbper': 'Planet.Period',
    'pl_orbpererr1': 'Planet.Period.Error.Upper',
    'pl_orbpererr2': 'Planet.Period.Error.Lower',
    'pl_orbper_reflink': 'Planet.Period.Reference',

    'pl_orbsmax': 'Planet.SemiMajorAxis',
    'pl_orbsmaxerr1': 'Planet.SemiMajorAxis.Error.Upper',
    'pl_orbsmaxerr2': 'Planet.SemiMajorAxis.Error.Lower',
    'pl_orbsmax_reflink': 'Planet.SemiMajorAxis.Reference',

    'pl_angsep': 'Planet.AngularSeparation',
    'pl_angseperr1': 'Planet.AngularSeparation.Error.Upper',
    'pl_angseperr2': 'Planet.AngularSeparation.Error.Lower',
    'pl_angsep_reflink': 'Planet.AngularSeparation.Reference',

    'pl_rade': 'Planet.RadiusEarth',
    'pl_radeerr1': 'Planet.RadiusEarth.Error.Upper',
    'pl_radeerr2': 'Planet.RadiusEarth.Error.Lower',
    'pl_rade_reflink': 'Planet.RadiusEarth.Reference',

    'pl_radj': 'Planet.RadiusJupiter',
    'pl_radjerr1': 'Planet.RadiusJupiter.Error.Upper',
    'pl_radjerr2': 'Planet.RadiusJupiter.Error.Lower',
    'pl_radj_reflink': 'Planet.RadiusJupiter.Reference',

    'pl_masse': 'Planet.MassEarth',
    'pl_masseerr1': 'Planet.MassEarth.Error.Upper',
    'pl_masseerr2': 'Planet.MassEarth.Error.Lower',
    'pl_masse_reflink': 'Planet.MassEarth.Reference',

    'pl_massj': 'Planet.MassJupiter',
    'pl_massjerr1': 'Planet.MassJupiter.Error.Upper',
    'pl_massjerr2': 'Planet.MassJupiter.Error.Lower',
    'pl_massj_reflink': 'Planet.MassJupiter.Reference',
    
    'pl_msinie': 'Planet.MinimumMassEarth',
    'pl_msinieerr1': 'Planet.MinimumMassEarth.Error.Upper',
    'pl_msinieerr2': 'Planet.MinimumMassEarth.Error.Lower',
    'pl_msinie_reflink': 'Planet.MinimumMassEarth.Reference',

    'pl_msinij': 'Planet.MinimumMassJupiter',
    'pl_msinijerr1': 'Planet.MinimumMassJupiter.Error.Upper',
    'pl_msinijerr2': 'Planet.MinimumMassJupiter.Error.Lower',
    'pl_msinij_reflink': 'Planet.MinimumMassJupiter.Reference',
    
    'pl_bmasse': 'Planet.BestMassEstimateEarth',
    'pl_bmasseerr1': 'Planet.BestMassEstimateEarth.Error.Upper',
    'pl_bmasseerr2': 'Planet.BestMassEstimateEarth.Error.Lower',
    'pl_bmasse_reflink': 'Planet.BestMassEstimateEarth.Reference',

    'pl_bmassj': 'Planet.BestMassEstimateJupiter',
    'pl_bmassjerr1': 'Planet.BestMassEstimateJupiter.Error.Upper',
    'pl_bmassjerr2': 'Planet.BestMassEstimateJupiter.Error.Lower',
    'pl_bmassj_reflink': 'Planet.BestMassEstimateJupiter.Reference',
    
    'pl_dens': 'Planet.Density',
    'pl_denserr1': 'Planet.Density.Error.Upper',
    'pl_denserr2': 'Planet.Density.Error.Lower',
    'pl_dens_reflink': 'Planet.Density.Reference',
    
    'pl_orbeccen': 'Planet.Eccentricity',
    'pl_orbeccenerr1': 'Planet.Eccentricity.Error.Upper',
    'pl_orbeccenerr2': 'Planet.Eccentricity.Error.Lower',
    'pl_orbeccen_reflink': 'Planet.Eccentricity.Reference',
    
    'pl_insol': 'Planet.InsolationFlux',
    'pl_insolerr1': 'Planet.InsolationFlux.Error.Upper',
    'pl_insolerr2': 'Planet.InsolationFlux.Error.Lower',
    'pl_insol_reflink': 'Planet.InsolationFlux.Reference',
    
    'pl_eqt': 'Planet.EquilibriumTemperature',
    'pl_eqterr1': 'Planet.EquilibriumTemperature.Error.Upper',
    'pl_eqterr2': 'Planet.EquilibriumTemperature.Error.Lower',
    'pl_eqt_reflink': 'Planet.EquilibriumTemperature.Reference',

    'pl_orbincl': 'Planet.Inclination',
    'pl_orbinclerr1': 'Planet.Inclination.Error.Upper',
    'pl_orbinclerr2': 'Planet.Inclination.Error.Lower',
    'pl_orbincl_reflink': 'Planet.Inclination.Reference',

    'pl_tranmid': 'Planet.TransitMidpoint',
    'pl_tranmiderr1': 'Planet.TransitMidpoint.Error.Upper',
    'pl_tranmiderr2': 'Planet.TransitMidpoint.Error.Lower',
    'pl_tranmid_reflink': 'Planet.TransitMidpoint.Reference',

    'pl_imppar': 'Planet.ImpactParameter',
    'pl_impparerr1': 'Planet.ImpactParameter.Error.Upper',
    'pl_impparerr2': 'Planet.ImpactParameter.Error.Lower',
    'pl_imppar_reflink': 'Planet.ImpactParameter.Reference',

    'pl_trandep': 'Planet.TransitDepth',
    'pl_trandeperr1': 'Planet.TransitDepth.Error.Upper',
    'pl_trandeperr2': 'Planet.TransitDepth.Error.Lower',
    'pl_trandep_reflink': 'Planet.TransitDepth.Reference',

    'pl_trandur': 'Planet.TransitDuration',
    'pl_trandurerr1': 'Planet.TransitDuration.Error.Upper',
    'pl_trandurerr2': 'Planet.TransitDuration.Error.Lower',
    'pl_trandur_reflink': 'Planet.TransitDuration.Reference',

    'pl_ratdor': 'Planet.RatioSemiMajorAxisToStellarRadius',
    'pl_ratdorerr1': 'Planet.RatioSemiMajorAxisToStellarRadius.Error.Upper',
    'pl_ratdorerr2': 'Planet.RatioSemiMajorAxisToStellarRadius.Error.Lower',
    'pl_ratdor_reflink': 'Planet.RatioSemiMajorAxisToStellarRadius.Reference',

    'pl_ratror': 'Planet.RatioPlanetRadiusToStellarRadius',
    'pl_ratrorerr1': 'Planet.RatioPlanetRadiusToStellarRadius.Error.Upper',
    'pl_ratrorerr2': 'Planet.RatioPlanetRadiusToStellarRadius.Error.Lower',
    'pl_ratror_reflink': 'Planet.RatioPlanetRadiusToStellarRadius.Reference',

    'pl_occdep': 'Planet.OccultationDepth',
    'pl_occdeperr1': 'Planet.OccultationDepth.Error.Upper',
    'pl_occdeperr2': 'Planet.OccultationDepth.Error.Lower',
    'pl_occdep_reflink': 'Planet.OccultationDepth.Reference',

    'pl_orbtper': 'Planet.EpochOfPeriastron',
    'pl_orbtpererr1': 'Planet.EpochOfPeriastron.Error.Upper',
    'pl_orbtpererr2': 'Planet.EpochOfPeriastron.Error.Lower',
    'pl_orbtper_reflink': 'Planet.EpochOfPeriastron.Reference',

    'pl_orblper': 'Planet.ArgumentOfPeriastron',
    'pl_orblpererr1': 'Planet.ArgumentOfPeriastron.Error.Upper',
    'pl_orblpererr2': 'Planet.ArgumentOfPeriastron.Error.Lower',
    'pl_orblper_reflink': 'Planet.ArgumentOfPeriastron.Reference',

    'pl_rvamp': 'Planet.RadialVelocityAmplitude',
    'pl_rvamperr1': 'Planet.RadialVelocityAmplitude.Error.Upper',
    'pl_rvamperr2': 'Planet.RadialVelocityAmplitude.Error.Lower',
    'pl_rvamp_reflink': 'Planet.RadialVelocityAmplitude.Reference',

    'pl_projobliq': 'Planet.ProjectedObliquity',
    'pl_projobliqerr1': 'Planet.ProjectedObliquity.Error.Upper',
    'pl_projobliqerr2': 'Planet.ProjectedObliquity.Error.Lower',
    'pl_projobliq_reflink': 'Planet.ProjectedObliquity.Reference',

    'pl_trueobliq': 'Planet.TrueObliquity',
    'pl_trueobliqerr1': 'Planet.TrueObliquity.Error.Upper',
    'pl_trueobliqerr2': 'Planet.TrueObliquity.Error.Lower',
    'pl_trueobliq_reflink': 'Planet.TrueObliquity.Reference',

    # Star parameters
    'st_refname': 'Star.Reference',
    'st_spectype': 'Star.Type',
    'st_spectype': 'Star.Type.Reference',
    
    'st_teff': 'Star.EffectiveTemperature',
    'st_tefferr1': 'Star.EffectiveTemperature.Error.Upper',
    'st_tefferr2': 'Star.EffectiveTemperature.Error.Lower',
    'st_teff_reflink': 'Star.EffectiveTemperature.Reference',
    
    'st_rad': 'Star.Radius',
    'st_raderr1': 'Star.Radius.Error.Upper',
    'st_raderr2': 'Star.Radius.Error.Lower',
    'st_rad_reflink': 'Star.Radius.Reference',
    
    'st_mass': 'Star.Mass',
    'st_masserr1': 'Star.Mass.Error.Upper',
    'st_masserr2': 'Star.Mass.Error.Lower',
    'st_mass_reflink': 'Star.Mass.Reference',

    'st_met': 'Star.Metallicity',
    'st_meterr1': 'Star.Metallicity.Error.Upper',
    'st_meterr2': 'Star.Metallicity.Error.Lower',
    'st_met_reflink': 'Star.Metallicity.Reference',

    'st_metratio': 'Star.MetallicityRatio',
    'st_metratioerr1': 'Star.MetallicityRatio.Error.Upper',
    'st_metratioerr2': 'Star.MetallicityRatio.Error.Lower',
    'st_metratio_reflink': 'Star.MetallicityRatio.Reference',

    'st_lum': 'Star.Luminosity',
    'st_lumerr1': 'Star.Luminosity.Error.Upper',
    'st_lumerr2': 'Star.Luminosity.Error.Lower',
    'st_lum_reflink': 'Star.Luminosity.Reference',
    
    'st_logg': 'Star.Logg',
    'st_loggerr1': 'Star.Logg.Error.Upper',
    'st_loggerr2': 'Star.Logg.Error.Lower',
    'st_logg_reflink': 'Star.Logg.Reference',

    'st_age': 'Star.Age',
    'st_ageerr1': 'Star.Age.Error.Upper',
    'st_ageerr2': 'Star.Age.Error.Lower',
    'st_age_reflink': 'Star.Age.Reference',

    'st_dens': 'Star.Density',
    'st_denserr1': 'Star.Density.Error.Upper',
    'st_denserr2': 'Star.Density.Error.Lower',
    'st_dens_reflink': 'Star.Density.Reference',

    'st_vsin': 'Star.RotationalVelocity',
    'st_vsinerr1': 'Star.RotationalVelocity.Error.Upper',
    'st_vsinerr2': 'Star.RotationalVelocity.Error.Lower',
    'st_vsin_reflink': 'Star.RotationalVelocity.Reference',

    'st_rotp': 'Star.RotationalPeriod',
    'st_rotperr1': 'Star.RotationalPeriod.Error.Upper',
    'st_rotperr2': 'Star.RotationalPeriod.Error.Lower',
    'st_rotp_reflink': 'Star.RotationalPeriod.Reference',

    # Technically under star, moved to System
    'st_radv': 'System.Velocity',
    'st_radverr1': 'System.Velocity.Error.Upper',
    'st_radverr2': 'System.Velocity.Error.Lower',
    'st_radv_reflink': 'System.Velocity.Reference',

    # System parameters
    'sy_refname': 'System.Reference',
    
    'sy_pm': 'System.TotalProperMotion',
    'sy_pmerr1': 'System.TotalProperMotion.Error.Upper',
    'sy_pmerr2': 'System.TotalProperMotion.Error.Lower',
    'sy_pm_reflink': 'System.TotalProperMotion.Reference',

    'sy_pmra': 'System.ProperMotionRightAscension',
    'sy_pmraerr1': 'System.ProperMotionRightAscension.Error.Upper',
    'sy_pmraerr2': 'System.ProperMotionRightAscension.Error.Lower',
    'sy_pmra_reflink': 'System.ProperMotionRightAscension.Reference',

    'sy_pmdec': 'System.ProperMotionDeclination',
    'sy_pmdecerr1': 'System.ProperMotionDeclination.Error.Upper',
    'sy_pmdecerr2': 'System.ProperMotionDeclination.Error.Lower',
    'sy_pmdec_reflink': 'System.ProperMotionDeclination.Reference',

    'sy_dist': 'System.Distance',
    'sy_disterr1': 'System.Distance.Error.Upper',
    'sy_disterr2': 'System.Distance.Error.Lower',
    'sy_dist_reflink': 'System.Distance.Reference',

    'sy_plx': 'System.Parallax',
    'sy_plxerr1': 'System.Parallax.Error.Upper',
    'sy_plxerr2': 'System.Parallax.Error.Lower',
    'sy_plx_reflink': 'System.Parallax.Reference',
    
    # Position keywords
    'rastr': 'Position.RightAscensionString',
    'decst': 'Position.DeclinationString',

    'ra': 'Position.RightAscension',
    'raerr1': 'Position.RightAscension.Error.Upper',
    'raerr2': 'Position.RightAscension.Error.Lower',
    'ra_reflink': 'Position.RightAscension.Reference',

    'dec': 'Position.Declination',
    'decerr1': 'Position.Declination.Error.Upper',
    'decerr2': 'Position.Declination.Error.Lower',
    'dec_reflink': 'Position.Declination.Reference',

    'glat': 'Position.GalacticLatitude',
    'glaterr1': 'Position.GalacticLatitude.Error.Upper',
    'glaterr2': 'Position.GalacticLatitude.Error.Lower',
    'glat_reflink': 'Position.GalacticLatitude.Reference',

    'glon': 'Position.GalacticLongitude',
    'glonerr1': 'Position.GalacticLongitude.Error.Upper',
    'glonerr2': 'Position.GalacticLongitude.Error.Lower',
    'glon_reflink': 'Position.GalacticLongitude.Reference',

    'elat': 'Position.EclipticLatitude',
    'elaterr1': 'Position.EclipticLatitude.Error.Upper',
    'elaterr2': 'Position.EclipticLatitude.Error.Lower',
    'elat_reflink': 'Position.EclipticLatitude.Reference',

    'elon': 'Position.EclipticLongitude',
    'elonerr1': 'Position.EclipticLongitude.Error.Upper',
    'elonerr2': 'Position.EclipticLongitude.Error.Lower',
    'elon_reflink': 'Position.EclipticLongitude.Reference',
    
    # Magnitude information
    'sy_bmag': 'Magnitude.B',
    'sy_bmagerr1': 'Magnitude.B.Error.Upper',
    'sy_bmagerr2': 'Magnitude.B.Error.Lower',
    'sy_bmag_reflink': 'Magnitude.B.Reference',

    'sy_vmag': 'Magnitude.V',
    'sy_vmagerr1': 'Magnitude.V.Error.Upper',
    'sy_vmagerr2': 'Magnitude.V.Error.Lower',
    'sy_vmag_reflink': 'Magnitude.V.Reference',

    'sy_jmag': 'Magnitude.J',
    'sy_jmagerr1': 'Magnitude.J.Error.Upper',
    'sy_jmagerr2': 'Magnitude.J.Error.Lower',
    'sy_jmag_reflink': 'Magnitude.J.Reference',

    'sy_hmag': 'Magnitude.H',
    'sy_hmagerr1': 'Magnitude.H.Error.Upper',
    'sy_hmagerr2': 'Magnitude.H.Error.Lower',
    'sy_hmag_reflink': 'Magnitude.H.Reference',

    'sy_kmag': 'Magnitude.K',
    'sy_kmagerr1': 'Magnitude.K.Error.Upper',
    'sy_kmagerr2': 'Magnitude.K.Error.Lower',
    'sy_kmag_reflink': 'Magnitude.K.Reference',

    'sy_umag': 'Magnitude.u',
    'sy_umagerr1': 'Magnitude.u.Error.Upper',
    'sy_umagerr2': 'Magnitude.u.Error.Lower',
    'sy_umag_reflink': 'Magnitude.u.Reference',

    'sy_gmag': 'Magnitude.g',
    'sy_gmagerr1': 'Magnitude.g.Error.Upper',
    'sy_gmagerr2': 'Magnitude.g.Error.Lower',
    'sy_gmag_reflink': 'Magnitude.g.Reference',

    'sy_rmag': 'Magnitude.r',
    'sy_rmagerr1': 'Magnitude.r.Error.Upper',
    'sy_rmagerr2': 'Magnitude.r.Error.Lower',
    'sy_rmag_reflink': 'Magnitude.r.Reference',

    'sy_imag': 'Magnitude.i',
    'sy_imagerr1': 'Magnitude.i.Error.Upper',
    'sy_imagerr2': 'Magnitude.i.Error.Lower',
    'sy_imag_reflink': 'Magnitude.i.Reference',

    'sy_zmag': 'Magnitude.z',
    'sy_zmagerr1': 'Magnitude.z.Error.Upper',
    'sy_zmagerr2': 'Magnitude.z.Error.Lower',
    'sy_zmag_reflink': 'Magnitude.z.Reference',

    'sy_w1mag': 'Magnitude.W1',
    'sy_w1magerr1': 'Magnitude.W1.Error.Upper',
    'sy_w1magerr2': 'Magnitude.W1.Error.Lower',
    'sy_w1mag_reflink': 'Magnitude.W1.Reference',

    'sy_w2mag': 'Magnitude.W2',
    'sy_w2magerr1': 'Magnitude.W2.Error.Upper',
    'sy_w2magerr2': 'Magnitude.W2.Error.Lower',
    'sy_w2mag_reflink': 'Magnitude.W2.Reference',

    'sy_w3mag': 'Magnitude.W3',
    'sy_w3magerr1': 'Magnitude.W3.Error.Upper',
    'sy_w3magerr2': 'Magnitude.W3.Error.Lower',
    'sy_w3mag_reflink': 'Magnitude.W3.Reference',

    'sy_w4mag': 'Magnitude.W4',
    'sy_w4magerr1': 'Magnitude.W4.Error.Upper',
    'sy_w4magerr2': 'Magnitude.W4.Error.Lower',
    'sy_w4mag_reflink': 'Magnitude.W4.Reference',

    'sy_gaiamag': 'Magnitude.Gaia',
    'sy_gaiamagerr1': 'Magnitude.Gaia.Error.Upper',
    'sy_gaiamagerr2': 'Magnitude.Gaia.Error.Lower',
    'sy_gaiamag_reflink': 'Magnitude.Gaia.Reference',

    'sy_icmag': 'Magnitude.IC',
    'sy_icmagerr1': 'Magnitude.IC.Error.Upper',
    'sy_icmagerr2': 'Magnitude.IC.Error.Lower',
    'sy_icmag_reflink': 'Magnitude.IC.Reference',

    'sy_tmag': 'Magnitude.TESS',
    'sy_tmagerr1': 'Magnitude.TESS.Error.Upper',
    'sy_tmagerr2': 'Magnitude.TESS.Error.Lower',
    'sy_tmag_reflink': 'Magnitude.TESS.Reference',

    'sy_kepmag': 'Magnitude.Kepler',
    'sy_kepmagerr1': 'Magnitude.Kepler.Error.Upper',
    'sy_kepmagerr2': 'Magnitude.Kepler.Error.Lower',
    'sy_kepmag_reflink': 'Magnitude.Kepler.Reference',

}

LABEL_MAPPER = {
    'Planet.Name': 'Planet name',
    'Star.Name': 'Host star name',
    'Planet.Letter': 'Planet letter',
    
    'Star.Catalogue.HD': 'HD catalog',
    'Star.Catalogue.HIP': 'HIP catalog',
    'Star.Catalogue.TIC': 'TIC catalog',
    'Star.Catalogue.GAIA': 'Gaia catalog',

    'System.NumberOfStars': 'Number of stars',
    'System.NumberOfPlanets': 'Number of planets',
    'System.NumberOfMoons': 'Number of moons',

    'Discovery.Method': 'Discovery method',
    'Discovery.Year': 'Discovery year',
    'Discovery.PublicationDate': 'Publication date',
    'Discovery.Locale': 'Location of discovery',
    'Discovery.Facility': 'Facility',
    'Discovery.Telescope': 'Telescope',
    'Discovery.Instrument': 'Instrument',

    'Flag.BinarySystem': 'Binary system',
    'Flag.RadialVelocity': 'Detected by RV',
    'Flag.PulsarTimingVariations': 'Detection by pulsar timing variations',
    'Flag.PulsationTimingVariations': 'Detection by pulsation timing variations',
    'Flag.Transit': 'Detection by transit',
    'Flag.AstrometricVariations': 'Detection by astrometric variations',
    'Flag.OrbitalBrightnessModulations': 'Detection by orbital brightness modulations',
    'Flag.Microlensing': 'Detection by microlensing',
    'Flag.EclipseTimingVariations': 'Detection by eclipse timing variations',
    'Flag.Imaging': 'Detection by imaging',
    'Flag.DiskKinematics': 'Detection by disk kinematics',

    'Planet.Period': 'Orbital period [days]',
    'Planet.SemiMajorAxis': 'Semi-major axis [AU]',
    'Planet.AngularSeparation': 'Angular separation [arcsec]',
    'Planet.RadiusEarth': 'Planet radius [Earth radii]',
    'Planet.RadiusJupiter': 'Planet radius [Jupiter radii]',
    'Planet.MassEarth': 'Planet mass [Earth masses]',
    'Planet.MassJupiter': 'Planet mass [Jupiter masses]',
    'Planet.MinimumMassEarth': 'Planet minimum mass [Earth masses]',
    'Planet.MinimumMassJupiter': 'Planet minimum mass [Jupiter masses]',
    'Planet.BestMassEstimateEarth': 'Planet best mass estimate [Earth masses]',
    'Planet.BestMassEstimateJupiter': 'Planet best mass estimate [Jupiter masses]',
    'Planet.Density': 'Planet density [g/cm^3]',
    'Planet.Eccentricity': 'Planet eccentricity',
    'Planet.InsolationFlux': 'Planet insolation flux [Earth flux]',
    'Planet.EquilibriumTemperature': 'Planet equilibrium temperature [K]',
    'Planet.Inclination': 'Planet inclination [deg]',
    'Planet.TransitMidpoint': 'Planet transit midpoint [BJD]',
    'Planet.ImpactParameter': 'Planet impact parameter',
    'Planet.TransitDepth': 'Planet transit depth',
    'Planet.TransitDuration': 'Planet transit duration [hours]',
    'Planet.RatioSemiMajorAxisToStellarRadius': 'Planet semi-major axis to stellar radius ratio',
    'Planet.RatioPlanetRadiusToStellarRadius': 'Planet radius to stellar radius ratio',
    'Planet.OccultationDepth': 'Planet occultation depth',
    'Planet.EpochOfPeriastron': 'Planet epoch of periastron [BJD]',
    'Planet.ArgumentOfPeriastron': 'Planet argument of periastron [deg]',
    'Planet.RadialVelocityAmplitude': 'Planet radial velocity amplitude [m/s]',
    'Planet.ProjectedObliquity': 'Planet projected obliquity [deg]',
    'Planet.TrueObliquity': 'Planet true obliquity [deg]',

    'Star.EffectiveTemperature': 'Stellar effective temperature [K]',
    'Star.Radius': 'Stellar radius [Solar radii]',
    'Star.Mass': 'Stellar mass [Solar masses]',
    'Star.Metallicity': 'Stellar metallicity',
    'Star.MetallicityRatio': 'Stellar metallicity ratio',
    'Star.Luminosity': 'Sterllar luminosity [Solar luminosities]',
    'Star.Logg': 'Stellar logg',
    'Star.Age': 'Stellar age [Gyr]',
    'Star.Density': 'Stellar density [g/cm^3]',
    'Star.RotationalVelocity': 'Stellar rotational velocity [km/s]',
    'Star.RotationalPeriod': 'Stellar rotational period [days]',

    'System.Velocity': 'Systemic velocity [km/s]',
    'System.TotalProperMotion': 'Systemic total proper motion [arcsec/yr]',
    'System.ProperMotionRightAscension': 'Systemic proper motion in right ascension [arcsec/yr]',
    'System.ProperMotionDeclination': 'Systemic proper motion in declination [arcsec/yr]',
    'System.Distance': 'System distance [pc]',
    'System.Parallax': 'System parallax [arcsec]',

    'Position.RightAscensionString': 'Right ascension',
    'Position.DeclinationString': 'Declination',
    'Position.RightAscension': 'Right ascension [deg]',
    'Position.Declination': 'Declination [deg]',
    'Position.GalacticLatitude': 'Galactic latitude [deg]',
    'Position.GalacticLongitude': 'Galactic longitude [deg]',
    'Position.EclipticLatitude': 'Ecliptic latitude [deg]',
    'Position.EclipticLongitude': 'Ecliptic longitude [deg]',

    'Magnitude.B': 'B Magnitude',
    'Magnitude.V': 'V Magnitude',
    'Magnitude.J': 'J Magnitude',
    'Magnitude.H': 'H Magnitude',
    'Magnitude.K': 'K Magnitude',
    'Magnitude.u': 'u Magnitude',
    'Magnitude.g': 'g Magnitude',
    'Magnitude.r': 'r Magnitude',
    'Magnitude.i': 'i Magnitude',
    'Magnitude.z': 'z Magnitude',
    'Magnitude.W1': 'W1 Magnitude',
    'Magnitude.W2': 'W2 Magnitude',
    'Magnitude.W3': 'W3 Magnitude',
    'Magnitude.W4': 'W4 Magnitude',
    'Magnitude.Gaia': 'Gaia Magnitude',
    'Magnitude.IC': 'IC Magnitude',
    'Magnitude.TESS': 'TESS Magnitude',
    'Magnitude.Kepler': 'Kepler Magnitude',
}


SCALE_MAPPER = {
    'Planet.Name': 'linear',
    'Star.Name': 'linear',
    'Planet.Letter': 'linear',
    
    'Star.Catalogue.HD': 'linear',
    'Star.Catalogue.HIP': 'linear',
    'Star.Catalogue.TIC': 'linear',
    'Star.Catalogue.GAIA': 'linear',
    
    'System.NumberOfStars': 'linear',
    'System.NumberOfPlanets': 'linear',
    'System.NumberOfMoons': 'linear',
    
    'Discovery.Method': 'linear',
    'Discovery.Year': 'linear',
    'Discovery.PublicationDate': 'linear',
    'Discovery.Locale': 'linear',
    'Discovery.Facility': 'linear',
    'Discovery.Telescope': 'linear',
    'Discovery.Instrument': 'linear',
    
    'Flag.BinarySystem': 'linear',
    'Flag.RadialVelocity': 'linear',
    'Flag.PulsarTimingVariations': 'linear',
    'Flag.PulsationTimingVariations': 'linear',
    'Flag.Transit': 'linear',
    'Flag.AstrometricVariations': 'linear',
    'Flag.OrbitalBrightnessModulations': 'linear',
    'Flag.Microlensing': 'linear',
    'Flag.EclipseTimingVariations': 'linear',
    'Flag.Imaging': 'linear',
    'Flag.DiskKinematics': 'linear',
    
    'Planet.Period': 'log',
    'Planet.SemiMajorAxis': 'log',
    'Planet.AngularSeparation': 'linear',
    'Planet.RadiusEarth': 'log',
    'Planet.RadiusJupiter': 'log',
    'Planet.MassEarth': 'log',
    'Planet.MassJupiter': 'log',
    'Planet.MinimumMassEarth': 'log',
    'Planet.MinimumMassJupiter': 'log',
    'Planet.BestMassEstimateEarth': 'log',
    'Planet.BestMassEstimateJupiter': 'log',
    'Planet.Density': 'linear',
    'Planet.Eccentricity': 'linear',
    'Planet.InsolationFlux': 'log',
    'Planet.EquilibriumTemperature': 'log',
    'Planet.Inclination': 'linear',
    'Planet.TransitMidpoint': 'linear',
    'Planet.ImpactParameter': 'linear',
    'Planet.TransitDepth': 'linear',
    'Planet.TransitDuration': 'linear',
    'Planet.RatioSemiMajorAxisToStellarRadius': 'linear',
    'Planet.RatioPlanetRadiusToStellarRadius': 'linear',
    'Planet.OccultationDepth': 'linear',
    'Planet.EpochOfPeriastron': 'linear',
    'Planet.ArgumentOfPeriastron': 'linear',
    'Planet.RadialVelocityAmplitude': 'linear',
    'Planet.ProjectedObliquity': 'linear',
    'Planet.TrueObliquity': 'linear',
    
    'Star.EffectiveTemperature': 'linear',
    'Star.Radius': 'linear',
    'Star.Mass': 'linear',
    'Star.Metallicity': 'linear',
    'Star.MetallicityRatio': 'linear',
    'Star.Luminosity': 'linear',
    'Star.Logg': 'linear',
    'Star.Age': 'linear',
    'Star.Density': 'linear',
    'Star.RotationalVelocity': 'linear',
    'Star.RotationalPeriod': 'linear',
    
    'System.Velocity': 'linear',
    'System.TotalProperMotion': 'linear',
    'System.ProperMotionRightAscension': 'linear',
    'System.ProperMotionDeclination': 'linear',
    'System.Distance': 'linear',
    'System.Parallax': 'linear',
    
    'Position.RightAscensionString': 'linear',
    'Position.DeclinationString': 'linear',
    'Position.RightAscension': 'linear',
    'Position.Declination': 'linear',
    'Position.GalacticLatitude': 'linear',
    'Position.GalacticLongitude': 'linear',
    'Position.EclipticLatitude': 'linear',
    'Position.EclipticLongitude': 'linear',
    
    'Magnitude.B': 'linear',
    'Magnitude.V': 'linear',
    'Magnitude.J': 'linear',
    'Magnitude.H': 'linear',
    'Magnitude.K': 'linear',
    'Magnitude.u': 'linear',
    'Magnitude.g': 'linear',
    'Magnitude.r': 'linear',
    'Magnitude.i': 'linear',
    'Magnitude.z': 'linear',
    'Magnitude.W1': 'linear',
    'Magnitude.W2': 'linear',
    'Magnitude.W3': 'linear',
    'Magnitude.W4': 'linear',
    'Magnitude.Gaia': 'linear',
    'Magnitude.IC': 'linear',
    'Magnitude.TESS': 'linear',
    'Magnitude.Kepler': 'linear',
}


from dataclasses import dataclass, field
import pandas as pd
import astropy.time as astime
import astropy.units as u
import logging
import csv
import matplotlib.pyplot as plt
import numpy as np
import os
import calplot
from matplotlib.colors import ListedColormap, BoundaryNorm
from ..utils.utilities import logger_default
from datetime import datetime, timedelta
from .observability import Event
from ..telescopes.telescopes import Telescope, VLT

logger = logging.getLogger(__name__)
if not logger.handlers:
    logger = logger_default(logger)


def define_baseline(table):
    array_length = len(table)
    table['Planet.Baseline'] = np.nanmax([np.nanmin([[3]*array_length, .75*table['Planet.TransitDuration']], axis=0), [
                                         2]*array_length],                                            axis=0)
    return table


@dataclass
class Windows:
    """
    A class to handle the generation and management of transit windows for exoplanet observations.

    Attributes
    ----------
    table : pd.DataFrame
        DataFrame containing the exoplanet data.
    center_phase : float, optional
        The center phase of the transit (default is 0.0).
    observing_period : astime.Time | None | str, optional
        The observing period for the transits (default is None).
    baseline : u.Quantity | None, optional
        The baseline for the transit windows (default is None).
    large_program : bool, optional
        Flag indicating if the program is a large program (default is False).
    directory : str, optional
        Directory to save output files (default is '').
    Airmass_limit : float, optional
        The airmass limit for observations (default is None).

    Methods
    -------
    generate_windows():
        Generate transit windows for the exoplanets in the table.
    print_windows():
        Print the transit windows for each exoplanet.
    __post_init__():
        Initialize the observing period and generate transit windows.
    get_proposal_period():
        Determine the observing period based on the provided input.
    _get_period_for_next_year():
        Set the observing period to the next year.
    _get_proposal_period_from_string():
        Parse the observing period from a string.
    get_dates_for_ESO_semester(P_number: int) -> astime.Time:
        Get the start and end dates for an ESO semester.
    to_eso_format(time_start, time_end, target_name, quality=1) -> str:
    generate_observability(location: Telescope, partial: float = 1, velocity_offset: None | float = None, velocity_range: float = 5, save_figures: bool = True):
    define_baseline():
    add_header_to_csv(csv_writer: csv.writer):
        Add a header row to a CSV file using the provided CSV writer.
    add_event_to_csv(csv_writer: csv.writer, event: Event):
        Add an event's details to a CSV file using the provided CSV writer.
    """

    table: pd.DataFrame
    center_phase: float = 0.0
    observing_period: astime.Time | None | str = None
    baseline: u.Quantity | None = None
    large_program: bool = False
    directory: str = ''
    Airmass_limit: float = None

    def generate_windows(self):
        """
        Generate transit windows for planets in the table.

        This method calculates the midpoints of transit windows for each planet in the table
        within the specified observing period. It also calculates the associated errors for
        these midpoints.
        Raises:
            ValueError: If the table is empty and no calculations can be performed.
        Returns:
            None
        """

        if len(self.table) == 0:
            raise ValueError('Table is empty. Cannot calculate windows')
        else:
            logger.info(
                f'About to calculate event midpoints for {len(self.table)} planets')

        Ti_values = []
        Ti_sigma_values = []

        T1, T2 = self.observing_period[0], self.observing_period[1]

        for _, row in self.table.iterrows():
            T_c = row['Planet.TransitMidpoint'] * u.day
            T_c_error = np.max([row['Planet.TransitMidpoint.Error.Lower'],
                               row['Planet.TransitMidpoint.Error.Upper']]) * u.day
            P = row['Planet.Period'] * u.day
            P_error = np.max([row['Planet.Period.Error.Lower'],
                             row['Planet.Period.Error.Upper']]) * u.day

            # Round up to next integer
            n_min = np.ceil(((T1 - T_c).jd)*u.day / P)
            # Round down to previous integer
            n_max = np.floor(((T2 - T_c).jd)*u.day / P)

            n_values = np.arange(n_min, n_max + 1, dtype=int)

            Ti = T_c + n_values * P
            sigma_values = np.sqrt(T_c_error**2 + n_values**2 * P_error**2)
            # Shouldn't change, but to be sure we are within the observing period
            valid_Ti = Ti[(Ti > T1.jd * u.day) & (Ti < T2.jd*u.day)]
            valid_sigma = sigma_values[(
                Ti > T1.jd*u.day) & (Ti < T2.jd*u.day)].to(u.min)

            valid_Ti = [astime.Time(t, format='jd') for t in valid_Ti]

            Ti_values.append(valid_Ti)
            Ti_sigma_values.append(valid_sigma)

        self.table['Planet.TransitWindowCenter'] = Ti_values
        self.table['Planet.TransitWindowCenter.Error'] = Ti_sigma_values

        return

    def print_windows(self):
        """
        Prints the transit windows for each planet in the table.

        Iterates over each row in the table and prints the transit window centers
        along with their uncertainties for each planet. The output is formatted
        with a separator line and includes the planet name, transit window center,
        uncertainty, and the formatted date and time in UT.

        Returns:
            None
        """

        for _, row in self.table.iterrows():
            logger.print('='*25)
            logger.print(f"Transit windows centers for {row['Planet.Name']}")
            for window, uncertainty in zip(row['Planet.TransitWindowCenter'], row['Planet.TransitWindowCenter.Error']):
                logger.print(
                    f"    {window} ± {uncertainty:.2f} | {window.strftime('%Y-%m-%d %H:%M')} UT")

    def __post_init__(self):
        """
        Post-initialization method to set up the observing period, generate windows, 
        and define the baseline. This method is automatically called after the 
        object's initialization.

        It performs the following steps:
        1. Retrieves the proposal period and logs the observing period.
        2. Generates observation windows.
        3. Defines the baseline for observations.
        """

        self.get_proposal_period()
        logger.info('='*25)
        logger.info('Set observing period:')
        logger.info(f'    {self.observing_period[0]}')
        logger.info(f'    {self.observing_period[1]}')
        logger.info('='*25)

        self.generate_windows()
        self.define_baseline()

    def get_proposal_period(self):
        """
        Determines the proposal period based on the observing period attribute.

        This method checks the type of the `observing_period` attribute and 
        determines the proposal period accordingly. If `observing_period` is 
        None, it calls `_get_period_for_next_year()`. If `observing_period` is 
        a string, it calls `_get_proposal_period_from_string()`. If 
        `observing_period` is of type `astime.Time`, it does nothing. For any 
        other type, it raises a `ValueError`.
        Raises:
            ValueError: If `observing_period` is not None, str, or astime.Time.
        """

        if self.observing_period is None:
            self._get_period_for_next_year()
        elif type(self.observing_period) == str:
            self._get_proposal_period_from_string()
        elif type(self.observing_period) == astime.Time:
            return
        else:
            raise ValueError('Invalid type for observing period.')

    def _get_period_for_next_year(self):
        """
        Calculate the observing period for the next year.
        This method sets the `observing_period` attribute to a time range
        starting from today at 12:00 PM to the same date and time next year.
        The dates are formatted as 'YYYY-MM-DD 12:00:00.000' and converted
        to an `astime.Time` object with UTC scale.

        Returns:
            None
        """

        today = datetime.now().replace(hour=12, minute=0, second=0, microsecond=0)
        next_year = today.replace(year=today.year + 1)

        date1 = today.strftime('%Y-%m-%d 12:00:00.000')
        date2 = next_year.strftime('%Y-%m-%d 12:00:00.000')

        self.observing_period = astime.Time([date1, date2], scale='utc')

    def _get_proposal_period_from_string(self):
        """
        Extracts the proposal period from the observing period string and sets the observing period
        to the corresponding dates for the ESO semester.
        If the observing period string starts with 'ESO.', it extracts the numeric part of the string,
        converts it to an integer, and uses it to get the dates for the ESO semester.

        Raises:
            ValueError: If the observing period string does not start with 'ESO.'.
        """

        if self.observing_period.startswith('ESO.'):
            self.observing_period = self.get_dates_for_ESO_semester(
                P_number=int(
                    ''.join(c for c in self.observing_period if c.isdigit())),
            )
        else:
            raise ValueError('Invalid string format.')

    def get_dates_for_ESO_semester(self,
                                   P_number: int,
                                   ) -> astime.Time:
        """
        Provide starting and ending date of the ESO semester.

        Parameters
        ----------
        P_number : int
            Which semester are we looking for.

        Returns
        -------
        time : astime.Time
            Starting and ending date of the semester.

        """
        if not (self.large_program):
            if P_number % 2 == 0:
                time = astime.Time([f'{1967+ int(P_number/2)}-10-01 12:00:00.000',
                                    f'{1968+int(P_number/2)}-04-01 12:00:00.000'], scale='utc')
            else:
                time = astime.Time([f'{1968 + int((P_number-1)/2)}-04-01 12:00:00.000',
                                    f'{1968+ int((P_number-1)/2)}-10-01 12:00:00.000'], scale='utc')
        else:
            if P_number % 2 == 0:
                time = astime.Time([f'{1967+ int(P_number/2)}-10-01 12:00:00.000',
                                    f'{1969+ int(P_number/2)}-10-01 12:00:00.000'], scale='utc')
            else:
                time = astime.Time([f'{1968 + int((P_number-1)/2)}-04-01 12:00:00.000',
                                    f'{1970 + int((P_number-1)/2)}-04-01 12:00:00.000'], scale='utc')
        return time

    def to_eso_format(time_start, time_end, target_name, quality=1):
        """
        Convert two astropy Time objects to ESO's between() format.

        Parameters
        ----------
        time_start : astropy.time.Time
            Start time
        time_end : astropy.time.Time
            End time
        target_name : str
            Name of the target (e.g., "K2-406 b partial")
        quality : int, optional
            Quality parameter (default=1)

        Returns
        -------
        str
            ESO formatted time string

        Examples
        --------
        >>> from astropy.time import Time
        >>> t1 = Time('2025-06-24T23:24:00')
        >>> t2 = Time('2025-06-25T06:16:00')
        >>> print(to_eso_format(t1, t2, "K2-406 b partial"))
        between(2025-06-24T23:24,2025-06-25T06:16,1,"K2-406 b partial")
        """
        # Convert to ISO format and remove seconds if present
        t1_str = time_start.iso.split('.')[0].replace(':00', '')
        t2_str = time_end.iso.split('.')[0].replace(':00', '')

        return f'between({t1_str},{t2_str},{quality},"{target_name}")'

    def generate_observability(self,
                               location: Telescope,
                               partial: float = 1,
                               velocity_offset: None | float = None,
                               velocity_range: float = 5,  # km/s,
                               save_figures: bool = True,
                               ):
        """
        Generate observability windows for transiting exoplanets.

        Parameters:
        -----------
        location : Telescope
            The telescope location for which to generate observability windows.
        partial : float, optional
            The fraction of the transit duration to consider for partial observability (default is 1).
        velocity_offset : None or float, optional
            The velocity offset to apply (default is None).
        velocity_range : float, optional
            The range of velocities to consider in km/s (default is 5).
        save_figures : bool, optional
            Whether to save figures generated during the process (default is True).

        Returns:
        --------
        None
        """

        if not (self.Airmass_limit):
            if location.name == 'Very Large Telescope (VLT)':
                logger.info('Setting airmass limit to 2.2.')
                self.Airmass_limit = 2.2
            else:
                logger.warning('No airmass limit set. Setting to 2')
                self.Airmass_limit = 2

        os.makedirs(os.path.join(self.directory, location.name), exist_ok=True)
        complete_summary = os.path.join(
            self.directory, location.name, 'complete_summary.csv')
        csvfile_complete = open(complete_summary, 'w', newline='')
        csv_writer_complete = csv.writer(csvfile_complete)
        self.add_header_to_csv(csv_writer_complete)

        for _, row in self.table.iterrows():
            logger.print('='*25)
            logger.print(f"Working on {row['Planet.Name']}")
            logger.print(
                f"    with Tc: {row['Planet.TransitMidpoint']}, P: {row['Planet.Period']} days and T14: {row['Planet.TransitDuration']} hours")
            logger.print('='*25)
            valid_events = []

            for window, window_uncertainty in zip(row['Planet.TransitWindowCenter'], row['Planet.TransitWindowCenter.Error']):
                new_event = Event(
                    Location=location,
                    TransitDuration=row['Planet.TransitDuration'],
                    baseline=row['Planet.Baseline'],
                    Night=window,
                    row=row,
                    directory=self.directory,
                    Uncertainty=window_uncertainty,
                    partial=partial,
                    Airmass_limit=self.Airmass_limit,
                    velocity_offset=velocity_offset,
                    velocity_range=velocity_range,
                    save_figures=save_figures,
                )

                if new_event.quality > 0:
                    valid_events.append(new_event)

            if valid_events:
                per_planet_summary = os.path.join(
                    self.directory, location.name, row['Planet.Name'].replace(' ', ''), 'planet_summary.csv')
                with open(per_planet_summary, 'w', newline='') as csvfile_per_planet:
                    csv_writer_per_planet = csv.writer(csvfile_per_planet)
                    self.add_header_to_csv(csv_writer_per_planet)

                    # Write data for all valid events
                    for event in valid_events:
                        self.add_event_to_csv(csv_writer_per_planet, event)
                        self.add_event_to_csv(csv_writer_complete, event)
        csvfile_complete.close()
        if save_figures:
            self.generate_calendar(complete_summary)

    def define_baseline(self):
        """
        Define the baseline for the transit windows.

        This method checks if the baseline attribute is None. If it is, it calls the 
        `define_baseline` function to set the baseline for the table. Otherwise, it 
        sets the 'Planet.Baseline' column in the table to the baseline value converted 
        to hours.

        Returns:
            None
        """

        if self.baseline is None:
            self.table = define_baseline(self.table)
        else:
            self.table['Planet.Baseline'] = self.baseline.to(u.hour)

        return

    def add_header_to_csv(self,
                          csv_writer: csv.writer):
        """
        Adds a header row to a CSV file using the provided CSV writer.

        Parameters:
        -----------
        csv_writer : csv.writer
            The CSV writer object used to write to the CSV file.

        The header row contains the following columns:
        - 'Planet Name'
        - 'Night'
        - 'Observation start'
        - 'Observation end'
        - 'Quality'
        - 'Period'
        - 'Transit midpoint'
        - 'Transit center'
        - 'Transit center error [min]'
        - 'SM mode observable'
        """
        csv_writer.writerow([
            'Planet Name',
            'Night',
            'Observation start',
            'Observation end',
            'Quality',
            'Period',
            'Transit midpoint',
            'Transit center',
            'Transit center error [min]',
            'SM mode observable'
        ])
        return

    def add_event_to_csv(self,
                         csv_writer: csv.writer,
                         event: Event):
        """
        Adds an event's details to a CSV file using the provided CSV writer.

        Parameters:
        -----------
        csv_writer : csv.writer
            The CSV writer object used to write to the CSV file.
        event : Event
            The event object containing details about the transit event.

        The following details are written to the CSV file:
        - Planet Name
        - Night (formatted as YYYYMMDD)
        - Start Time (formatted as HH:MM)
        - End Time (formatted as HH:MM)
        - Quality of the event
        - Planet's orbital period
        - Planet's transit midpoint
        - Night of the event
        - Uncertainty in minutes
        - Whether the transit is observable by Service Mode using ESO rules
        """

        csv_writer.writerow([
            event.row['Planet.Name'],  # Planet Name
            event.TimeArray.midnight.datetime.strftime(
                '%Y%m%d'),              # Night
            event.TimeArray.time_array[event.ObservationsFull.complete[0]].strftime(
                '%H:%M'),  # Start Time
            # End Time
            event.TimeArray.time_array[event.ObservationsFull.complete[-1]
                                       ].strftime('%H:%M'),
            event.quality,       # Quality
            event.row['Planet.Period'],
            event.row['Planet.TransitMidpoint'],
            event.Night,
            event.Uncertainty.to(u.min).value,
            len(event.ObservationsFull.Ingress) > 30,
        ])
        return

    def generate_calendar(self, complete_summary: str):

        start_calendar = self.observing_period[0].datetime
        end_calendar = self.observing_period[-1].datetime

        df = pd.read_csv(complete_summary)
        df['Night'] = pd.to_datetime(df['Night'], format='%Y%m%d')

        def create_event_calendar(events_df, date_column, start_date, end_date, ):

            events_series = events_df[date_column].value_counts()
            events_series.index = pd.to_datetime(events_series.index)
            date_range = pd.date_range(
                start=start_date, end=end_date, freq='D').normalize()
            events_series = events_series.reindex(date_range, fill_value=0)

            # Create custom colormap with grey for out-of-range dates
            colors = ['darkgreen', 'orange']  # lightgrey for out-of-range
            bounds = [-0.5, 5.5]  # or similar small range around 1

            custom_cmap = ListedColormap(colors)
            norm = BoundaryNorm(bounds, custom_cmap.N)

            # Plot the calendar
            fig, ax = calplot.calplot(
                events_series,
                cmap=custom_cmap,  # Red-Yellow-Green colormap
                fillcolor='black',  # Color for days without data
                daylabels='MTWTFSS',
                dayticks=[0, 1, 2, 3, 4, 5, 6],  # Show some day labels
                figsize=(16, 8),
                suptitle='Transits events',
                colorbar=False
            )
            return fig

        with plt.ioff():
            fig = create_event_calendar(
                df,
                'Night',
                start_calendar,
                end_calendar
            )
            fig.savefig(complete_summary.replace('csv', 'png'))


if __name__ == '__main__':
    import os
    from ..database.NASA_exoplanet_archive import NASA_Exoplanet_Archive_CompositeDefault

    os.chdir('/media/chamaeleontis/Observatory_main/Code/observations_transits/PTO/')
    test = NASA_Exoplanet_Archive_CompositeDefault()
    test.load_API_table(force_load=True)

    logger.print(
        f"Length before further filtering of the table: {test.table.shape[0]}")
    test.table = test.table[test.table['Magnitude.V'] < 10]
    test.table = test.table[test.table['Planet.RadiusEarth'] > 3]
    test.table = test.table[test.table['Planet.RadiusEarth'] < 8]
    test.table = test.table[test.table['Planet.Period'] < 30]
    # test.table = test.table[test.table['Planet.Name'].isin(['HD 209458 b', 'HD 189733 b', 'WASP-76 b', ])]

    logger.print(
        f"Length after further filtering of the table: {test.table.shape[0]}")

    Transits = Windows(
        table=test.table,
        directory='/media/chamaeleontis/Observatory_main/ESO_scheduling/PTO_developement/',
        large_program=False
    )

    Transits.print_windows()

    Transits.generate_observability(
        location=VLT,
        save_figures=True,
    )



import matplotlib.pyplot as plt
import seaborn as sns
from dataclasses import dataclass
import astropy.time as astime
import datetime
import pandas as pd
from ..telescopes.telescopes import Telescope
import astropy.units as u
import numpy as np
import matplotlib.dates as md
import logging
from ..utils.utilities import logger_default

logger = logging.getLogger(__name__)
if not logger.handlers:
    logger = logger_default(logger)

import numpy as np
import matplotlib.pyplot as plt
from matplotlib.collections import LineCollection
from matplotlib.dates import date2num
from matplotlib.colors import ListedColormap, BoundaryNorm
import numpy as np
import astropy.constants as con
from PyAstronomy import pyasl
import numpy as np


def calculate_planetary_rv(time, row):
    """
    Calculates the radial velocity of a planet based on its orbital parameters.

    This function computes the radial velocity of a planet using its orbital parameters and the time of observation. It extracts the necessary parameters from the provided dictionary and performs the calculations using Kepler's laws.

    Parameters
    ----------
    time : astropy.time.Time
        The time at which to calculate the radial velocity.
    row : dict
        A dictionary containing the orbital parameters of the planet and star. Expected keys are:
        - 'Planet.Period': Orbital period of the planet.
        - 'Planet.SemiMajorAxis': Semi-major axis of the planet's orbit.
        - 'Planet.Eccentricity': Eccentricity of the planet's orbit.
        - 'Planet.ArgumentOfPeriastron': Argument of periastron (in degrees).
        - 'Planet.TransitMidpoint': Time of the planet's transit midpoint.
        - 'Star.Mass': Mass of the star.
        - 'Planet.MassJupiter': Mass of the planet in Jupiter masses.
        - 'Planet.Inclination': Inclination of the planet's orbit (in degrees).
        - 'Omega_deg' (optional): Longitude of the ascending node (in degrees). Default is 0.0.

    Returns
    -------
    numpy.ndarray
        The radial velocity of the planet at the given time.
    """
    # Extract parameters with defaults for optional ones
    period = row['Planet.Period']
    semi_major_axis = row['Planet.SemiMajorAxis']
    eccentricity = row['Planet.Eccentricity']
    omega_deg = row['Planet.ArgumentOfPeriastron']
    t_transit = row['Planet.TransitMidpoint']
    m_star = row['Star.Mass']
    m_planet = row['Planet.MassJupiter'] * u.M_jup.to(u.M_sun)
    inclination_deg = row['Planet.Inclination']
    Omega_deg = row.get('Omega_deg', 0.0)
    t = time.jd
    omega = np.radians(omega_deg)
    # Calculate time of periastron from transit time
    f_transit = np.pi/2 - omega
    E_transit = 2 * np.arctan(np.sqrt((1-eccentricity)/(1+eccentricity)) * 
                             np.tan(f_transit/2))
    M_transit = E_transit - eccentricity * np.sin(E_transit)
    dt = M_transit * period / (2*np.pi)
    t_periastron = t_transit - dt

    # Create Keplerian orbit
    ke = pyasl.KeplerEllipse(semi_major_axis, period, e=eccentricity,
                            Omega=Omega_deg, i=inclination_deg, w=omega_deg)

    # Calculate orbital position
    vel = ke.xyzVel(t-t_periastron)[:,2] *u.au/u.day
    vel = vel * (u.au/u.day).to(u.m/u.s)
    # Extract radial velocity (z-component of velocity)
    mass_ratio = m_planet / (m_star + m_planet)
    star_vel = vel * mass_ratio
    planet_vel = star_vel * (-(m_star/m_planet))
    return planet_vel

@dataclass
class WindowPlot():
    def generate_plots(self):
        with plt.ioff():
            fig, ax = plt.subplots(1, figsize=(18, 12))
            self._plot_basic_setting(fig, ax)
            self._plot_twilight(ax)
            self._plot_Moon(ax)
            self._plot_target(ax)
            self._plot_observation(ax)
            self._plot_BERV_spine(ax)
            self._write_system_parameters(ax)
            return fig, ax

    def _plot_BERV_spine(self, ax):
        if self.velocity_offset is None:
            return
        
        try:
            self.row['System.Velocity']
            pass
        except:
            logger.warning('Systemic velocity not found. Setting to 0')
            self.row['System.Velocity'] = 0
        
        if np.isnan(self.row['System.Velocity']):
            self.row['System.Velocity'] = 0
        
        planet_rv = calculate_planetary_rv(self.TimeArray.time_array, self.row)
        
        # Get the top spine's path
        x_data = date2num(self.TimeArray.time_array.to_value(format='datetime'))
        values = abs((-self.BERV + self.row['System.Velocity']) - self.velocity_offset + planet_rv)

        _, ymax = ax.get_ylim()

        # Create points for the line segments
        points = np.array([x_data, np.ones_like(x_data) * ymax+0.05]).T.reshape(-1, 1, 2)
        segments = np.concatenate([points[:-1], points[1:]], axis=1)

        # Define colors for different ranges
        colors = ['red', 'yellow', 'green']
        cmap = ListedColormap(colors)

        # Define boundaries for color changes
        bounds = [0, self.velocity_range, self.velocity_range*2, 999]
        norm = BoundaryNorm(bounds, cmap.N)

        # Create LineCollection with discrete colors
        lc = LineCollection(segments, cmap=cmap, norm=norm, linewidth=20)
        lc.set_array(values[:-1])
        
        # Remove original top spine
        ax.spines['top'].set_visible(False)

        # Add colored line collection
        ax.add_collection(lc)
        cbar = plt.colorbar(lc)
        cbar.set_ticks([2.5, 7.5, (bounds[-1]-bounds[-2])/2])  # Center of each color range
        cbar.set_ticklabels(['Contamination in range', 'Contamination in 2x range', 'No contamination'])
        
        # raise NotImplementedError('Implement planet velocity properly')
        return 

    def _plot_contamination_in_transit(self, ax):
        
        
        
        
        ...

    def _plot_basic_setting(self,
                            fig: plt.Figure,
                            ax: plt.Axes):
        """
        Plot basic settings regardless of what planet is plotted.

        Parameters
        ----------
        fig : plt.Figure
            Figure on which to plot on.
        ax : plt.Axes
            Artist on which to plot on.

        Returns
        -------
        None.

        """
        Night = (self.TimeArray.midnight-1*u.day).to_value('datetime')

        ax.set_title(
            f"Transit of {self.row['Planet.Name']} on night: {Night.strftime('%Y%m%d')} at {self.Location.name} with precision on $T_c$ of: {self.Uncertainty.to(u.min):.2f}; Quality: {self.quality}", fontsize=14)

        ax.axhline(
            np.arcsin(1/self.Airmass_limit) * 180/np.pi,
            ls='--', lw=2,
            color='darkred'
        )

        ax.grid(color='black', linestyle='dashed', linewidth=0.3, alpha=1.0)
        xfmt = md.DateFormatter('%d/%m  %H')
        ax.xaxis.set_major_formatter(xfmt)
        fig.autofmt_xdate()
        hours = md.HourLocator()
        ax.xaxis.set_major_locator(hours)

        ax.set_ylim(0, 90)
        ax2 = ax.twinx()
        ax.set_ylabel('Altitude [deg]', color='k')
        ax2.set_ylabel('Airmass', color='k')
        ax2.tick_params('y', colors='k')
        ax2.set_ylim([0, 90])
        ax2.set_yticklabels(['', 5.76, 2.92, 2.00, 1.56,
                            1.31, 1.15, 1.06, 1.02, 1.00])

        ax.set_xlim(self.TimeArray.Sunset,
                    self.TimeArray.Sunrise)

    def _plot_twilight(self,
                       ax: plt.Axes):
        """
        Overplot the twilight area.

        Parameters
        ----------
        ax : plt.Axes
            Artist on which to plot on.

        Returns
        -------
        None.

        """
        ax.fill_between(self.TimeArray.time_array.to_value(format='datetime'),
                        0, 90,
                        self.AltitudeArray.Sun.alt.value < 0,
                        color='blue',
                        alpha=0.3
                        )
        ax.fill_between(self.TimeArray.time_array.to_value(format='datetime'),
                        0, 90,
                        self.AltitudeArray.Sun.alt.value < -18,
                        color='blue',
                        alpha=0.5
                        )
        return

    def _plot_Moon(self,
                   ax: plt.Axes):
        """
        Overplot the Moon position.

        Parameters
        ----------
        ax : plt.Axes
            Artist on which to plot on.

        Returns
        -------
        None.

        """
        ax.plot(self.TimeArray.time_array.to_value(format='datetime'),
                self.AltitudeArray.Moon.alt.value,
                ls='-',
                label='Moon',
                linewidth=2,
                color='yellow',
                alpha=0.7
                )
        return

    def _plot_target(self,
                     ax: plt.Axes):
        """
        Plot the target, highlighting the transit area.

        Parameters
        ----------
        ax : plt.Axes
            Artist on which to plot.

        Returns
        -------
        None.

        """
        ax.plot(self.TimeArray.time_array.to_value(format='datetime'),
                self.AltitudeArray.target.alt.value,
                ls='-',
                label='Target',
                linewidth=4,
                color='red',
                )
        ax.plot(self.TimeArray.time_array[self.Indices.Transit].to_value(format='datetime'),
                self.AltitudeArray.target[self.Indices.Transit].alt.value,
                ls='-',
                label='Target',
                linewidth=10,
                color='red',
                )

        ax.scatter(
            self.TimeArray.time_array[self.Indices.Transit[0][0]].to_value(
                format='datetime'),
            self.AltitudeArray.target[self.Indices.Transit[0][0]].alt.value,
            color='white',
            zorder=999,
            s=200,
        )
        ax.scatter(
            self.TimeArray.time_array[self.Indices.Transit[-1]
                                      [-1]].to_value(format='datetime'),
            self.AltitudeArray.target[self.Indices.Transit[-1][-1]].alt.value,
            color='white',
            zorder=999,
            s=200,
        )

        return

    def _plot_observation(self,
                          ax: plt.Axes):

        # CLEANME
        self.twilight_observation_time = [
                    self.TimeArray.time_array[self.ObservationsTwilight.complete[0]].to_value(
                        format='datetime'),
                    self.TimeArray.time_array[self.ObservationsTwilight.complete[-1]].to_value(
                        format='datetime')
                ]
                
        if len(self.ObservationsFull.complete) != 0:
            ax.fill_between(self.TimeArray.time_array[self.ObservationsFull.complete].to_value(format='datetime'),
                0, 10,
                color='lime',
                alpha=1,
            )

            ax.text((self.TimeArray.time_array[self.ObservationsFull.complete[0]]+30*u.min).to_value(format='datetime'),
                    5,
                    'Observations (no twilight): ' +
                    self.TimeArray.time_array[self.ObservationsFull.complete[0]].to_value(
                        format='datetime').strftime('%H:%M') + ' - ' +
                    self.TimeArray.time_array[self.ObservationsFull.complete[-1]].to_value(
                        format='datetime').strftime('%H:%M'),
                    fontsize=10,
                    )
            
            self.full_observation_time = [
                self.TimeArray.time_array[self.ObservationsFull.complete[0]].to_value(
                    format='datetime'),
                self.TimeArray.time_array[self.ObservationsFull.complete[-1]
                                          ].to_value(format='datetime')
            ]
            self.twilight_full_is_same = (self.full_observation_time == self.twilight_observation_time)
        else:
            self.twilight_full_is_same = False
            
        
        if (((self.FlagsFull.baseline_ingress) or  # Ingress baseline missing
            (self.FlagsFull.baseline_egress) or  # Egress baseline missing
                (not (self.FlagsWindow.visible) and
                 self.FlagsWindow.visible_twilight)) and # Transit into twilight
            not(self.twilight_full_is_same)): # Twilight == Full observations
            
            ax.fill_between(self.TimeArray.time_array[self.ObservationsTwilight.complete].to_value(format='datetime'),
                10, 20,
                color='orange',
                alpha=1,
            )

            ax.text((self.TimeArray.time_array[self.ObservationsTwilight.complete[0]]+30*u.min).to_value(
                format='datetime'),
                15,
                'Observation (with twilight): ' +
                self.TimeArray.time_array[self.ObservationsTwilight.complete[0]].to_value(
                format='datetime').strftime('%H:%M') + ' - ' +
                self.TimeArray.time_array[self.ObservationsTwilight.complete[-1]].to_value(
                format='datetime').strftime('%H:%M'),
                fontsize=10,
            )

            ax.axvline(self.TimeArray.time_array[self.ObservationsTwilight.complete[0]].to_value(format='datetime'),
                       ls='--',
                       lw=2,
                       color='orange'
                       )
            ax.axvline(self.TimeArray.time_array[self.ObservationsTwilight.complete[-1]].to_value(format='datetime'),
                       ls='--',
                       lw=2,
                       color='orange')


        ax.axvline(self.TimeArray.T1.to_value(format='datetime'),
                   ls='--', lw=2, color='black')
        ax.axvline(self.TimeArray.T4.to_value(format='datetime'),
                   ls='--', lw=2, color='black')
        if len(self.ObservationsFull.complete) != 0:
            ax.axvline(self.TimeArray.time_array[self.ObservationsFull.complete[0]].to_value(
                format='datetime'), ls='--', lw=2, color='lime')
            ax.axvline(self.TimeArray.time_array[self.ObservationsFull.complete[-1]].to_value(
                format='datetime'), ls='--', lw=2, color='lime')

        return

    def _write_system_parameters(self,
                                 ax: plt.Axes):
        """
        Write the system parameters for given system, including Right Ascension, Declination,V magnitude, Moon angle separation, Transit length, Period, ingress and egress timings.

        Parameters
        ----------
        ax : plt.Axes
            DESCRIPTION.

        Returns
        -------
        None.

        """
        from matplotlib import rc

        if self.AltitudeArray.MoonSeparation.value > 45:
            color_moon = 'white'
        elif self.AltitudeArray.MoonSeparation.value > 30:
            color_moon = 'orange'
        else:
            color_moon = 'red'

        format_text = {'style': 'italic',
                       'size': 12
                       }

        text = ax.text(self.TimeArray.Sunset, 97,
                       f"RA: {self.row['Position.RightAscension']:.2f} ",
                       **format_text
                       )

        text = ax.annotate(
            f"DEC: {self.row['Position.Declination']:.2f} ",
            xycoords=text,
            xy=(1, 0), verticalalignment="bottom",
            **format_text
        )

        text = ax.annotate(
            f"; V = {self.row['Magnitude.V']:.3f} ",
            xycoords=text,
            xy=(1, 0), verticalalignment="bottom",
            **format_text
        )
        text = ax.annotate(
            f"; Moon = {self.AltitudeArray.MoonSeparation.value:.0f} deg ",
            xycoords=text,
            xy=(1, 0), verticalalignment="bottom",
            bbox = dict(facecolor=color_moon, edgecolor=color_moon,
                    boxstyle='round,pad=1'),
            **format_text,
        )
        text = ax.annotate(
            '; $T_{14}$' + f" = {self.row['Planet.TransitDuration']:.3f} h ",
            xycoords=text,
            xy=(1, 0), verticalalignment="bottom",
            **format_text
        )

        text = ax.annotate(
            f"; P = {self.row['Planet.Period']:.5f} d ",
            xycoords=text,
            xy=(1, 0), verticalalignment="bottom",
            **format_text
        )
        
        if 'Flag.TransitTimingVariations' in self.row.keys():
            if self.row['Flag.TransitTimingVariations']:
                text = ax.annotate(
                    f"; TTV detected",
                    xycoords=text,
                    xy=(1, 0), verticalalignment="bottom",
                    bbox = dict(facecolor='red', edgecolor='red',
                    boxstyle='round,pad=1'),
                    **format_text
                )
        
        sunset_text = ax.text(self.TimeArray.Sunset, -10,
                              f"ingress= {self.TimeArray.T1.to_value('datetime').strftime('%H:%M')}",
                              style='italic',
                              bbox=dict(
                                  facecolor='none',
                                  edgecolor='black',
                                  pad=7,
                    boxstyle='round,pad=1'
                              )
                              )

        sunset_text = ax.annotate(
            (f"Observations (no twilight): {self.TimeArray.time_array[self.ObservationsFull.complete[0]].to_value(format='datetime').strftime('%H:%M')} " +
             f"- {self.TimeArray.time_array[self.ObservationsFull.complete[-1]].to_value(format='datetime').strftime('%H:%M')}"),
            xycoords=sunset_text,
            xy=(1.4, 0), verticalalignment="bottom",
            bbox=dict(
                facecolor='green',
                edgecolor='black',
                boxstyle='round, pad=1'
            )
        )

        if not(self.full_observation_time == self.twilight_observation_time):
            sunset_text = ax.annotate(
                (f"Observation (with twilight): {self.TimeArray.time_array[self.ObservationsTwilight.complete[0]].to_value(format='datetime').strftime('%H:%M')} " +
                f"- {self.TimeArray.time_array[self.ObservationsTwilight.complete[-1]].to_value(format='datetime').strftime('%H:%M')}"),
                xycoords=sunset_text,
                xy=(1.4, 0), verticalalignment="bottom",
                bbox=dict(
                    facecolor='orange',
                    edgecolor='black',
                    boxstyle='round,pad=1'
                ),
            )

        ax.text(self.TimeArray.Sunrise, -10,
                f"egress= {self.TimeArray.T4.to_value('datetime').strftime('%H:%M')}",
                style='italic',
                bbox=dict(
                    facecolor='none',
                    edgecolor='black',
                    pad=7)
                )
        return



import specutils as sp
import pandas as pd
import numpy as np
from astropy.modeling import models, fitting

def normalize_spectrum(spectrum: sp.Spectrum1D):
    normalization_function = pd.Series(spectrum.flux).rolling(10000,min_periods=1,center=True).quantile(0.85)
    new_spectrum = spectrum.divide(normalization_function.values * spectrum.flux.unit)
    return new_spectrum


# import specutils as sp
# import astropy
# import astropy.units as u
# import numpy as np
# import pandas as pd
# import os 


# # cwd = os.getcwd()
# # os.chdir('/media/chamaeleontis/Observatory_main/Code/')
# # from rats.utilities import default_logger_format
# # import rats.parameters as para
# # import rats.spectra_manipulation as sm
# # os.chdir(cwd)

# import calculations as calc
# import instruments as inst
# import plots as plot
# import logging


# logger = logging.getLogger(__name__)
# logger = default_logger_format(logger) 

# def _load_ETC_file(filename: str) -> np.ndarray:
#     """
#     Load the ETC files based on filename.

#     Parameters
#     ----------
#     filename : str
#         Filename location.

#     Returns
#     -------
#     SNR : np.ndarray
#         Array of [wavelength, expected SNR] as loaded from the file.

#     """
#     f = open(filename,'r')
    
#     SNR =[]
#     for line in f.readlines():
#         if line.startswith('#'):
#             continue
        
#         wavelength, snr = line.replace('\n','').split('\t')
        
#         SNR.append([float(wavelength), float(snr)])
    
#     return np.asarray(SNR)

# #%%
# def _find_stellar_type_ESPRESSO(T_eff: float) -> str:
#     """
#     Find stellar type based on effective temperature.

#     Parameters
#     ----------
#     T_eff : float
#         Effective temperature of the star in K.

#     Returns
#     -------
#     spectral_type : str
#         Spectral type closest to the true one that is available in ESPRESSO ETC.

#     """
#     if T_eff > 54000:
#         return 'O5'
#     elif T_eff > 43300:
#         return 'O9'
#     elif T_eff > 29200:
#         return 'B1'
#     elif T_eff > 23000:
#         return 'B3'
#     elif T_eff > 15000:
#         return 'B8'
#     elif T_eff > 11800:
#         return 'B9'
#     elif T_eff > 10000:
#         return 'A0'
#     elif T_eff > 9450:
#         return 'A1'
#     elif T_eff > 8480:
#         return 'F0'
#     elif T_eff > 5900:
#         return 'G0'
#     elif T_eff > 5560:
#         return 'G2'
#     elif T_eff > 4730:
#         return 'K2'
#     elif T_eff > 3865:
#         return 'K7'
#     else:
#         return 'M2'
    

# def _load_SNR_table_ESPRESSO(
#     TargetParameters: para.SystemParametersComposite,
#     instrument: str = 'ESPRESSO'
#     ) -> tuple[np.ndarray, np.ndarray, np.ndarray]:
#     """
#     Load SNR tables based on the system parameters and instrument used.

#     Parameters
#     ----------
#     system_parameters : pd.DataFrame
#         System parameters as provided by NASA archive.
#     instrument : str, optional
#         Which instrument to use. The default is 'ESPRESSO'.

#     Returns
#     -------
#     best_scenario : np.ndarray
#         Array of [wavelength, expected SNR] for best case scenario.
#     average_scenario : np.ndarray
#         Array of [wavelength, expected SNR] for average (50%) case scenario.
#     worst_scenario : np.ndarray
#         Array of [wavelength, expected SNR] for worst case scenario.

#     """
#     if instrument != 'ESPRESSO' or 'ESPRESSO_4UT':
#         logger.warning('Loading SNR values from ESPRESSO, even through usage of different instrument. This will increase error in the resulting dataset, which however will not show in the result.')
    
#     spectral_type = _find_stellar_type_ESPRESSO(
#         float(TargetParameters.Star.temperature.data)
#         )
    
#     location_of_ETC_files = 'PTO/./ETC_{instrument}/{spectral_type}/'.format(instrument = instrument, spectral_type = spectral_type)
    
#     best_scenario = _load_ETC_file(location_of_ETC_files + '0.5arcsec_900s.etc')
#     average_scenario = _load_ETC_file(location_of_ETC_files + '0.8arcsec_900s.etc')
#     worst_scenario = _load_ETC_file(location_of_ETC_files + '1.3arcsec_900s.etc')
    
#     return best_scenario, average_scenario, worst_scenario
# #%% Scaling by instrument
# def _scale_instrument(spectrum: sp.Spectrum1D,
#                       from_instrument:str,
#                       to_instrument: str):
#     """
#     Scale the SNR between instrument, using the telescope mirror size. Instrument must be within the Instruments_size Enum class.

#     Parameters
#     ----------
#     spectrum : sp.Spectrum1D
#         Spectrum to scale
#     from_instrument : str
#         Original instrument, usually ESPRESSO, for which the code was implemented.
#     to_instrument : str
#         Target instrument, for which we want to simulate the dataset.
#     """
    
#     scale_factor = (inst.Instruments_size[to_instrument].value / inst.Instruments_size[from_instrument].value)**2 
    
#     new_spectrum = sp.Spectrum1D(
#         spectral_axis = spectrum.spectral_axis,
#         flux = spectrum.flux * scale_factor,
#         uncertainty = astropy.nddata.StdDevUncertainty(
#             np.sqrt(spectrum.flux * scale_factor)
#             ),
#         mask = np.zeros_like(spectrum.flux),
#         meta= {}
#         )
    
#     return new_spectrum

# #%% vactoair
# def vactoair(wlnm: np.ndarray):
#     """
#     Change from vacuum to air wavelengths.

#     Parameters
#     ----------
#     wlnm : np.ndarray
#         Wavelenght grid in nanometers (actually invariant towards units)
#     """
#     wlA = wlnm*10.0
#     s = 1e4/wlA
#     f = 1.0 + 5.792105e-2/(238.0185e0 - s**2) + 1.67917e-3/( 57.362e0 - s**2)
#     return(wlA/f/10.0)

# #%%
# def scale_estimated_SNR_with_Vmag(scenario: np.ndarray,
#                                   TargetParameters: para.SystemParametersComposite) -> np.ndarray:
#     """
#     Scale estimated SNR with V magnitude of the target.

#     Parameters
#     ----------
#     scenario : np.ndarray
#         Scenario as loaded by load_SNR_table function.
#     system_parameters : para.SystemParametersComposite
#         System parameters as loaded by NASA archive using the rats.para.SystemParametersComposite class.

#     Returns
#     -------
#     scenario : np.ndarray
#         Adjusted SNR of given scenario.

#     """
#     scenario[:,1] *= np.sqrt(10**(0.4*(10 - TargetParameters.Star.magnitudes.V.data)))
#     return scenario

# #%%
# def _scale_spectrum_with_SNR(
#     spectrum: sp.Spectrum1D,
#     SNR: np.ndarray) -> sp.Spectrum1D:
#     """
#     Scale spectrum according to estimated SNR.

#     Parameters
#     ----------
#     spectrum : sp.Spectrum1D
#         Model spectrum to scale.
#     SNR : np.ndarray, optional
#         Estimated SNR of single exposure as provided by _load_SNR_table.

#     Returns
#     -------
#     spectrum : sp.Spectrum1D
#         Simulated exposure with given SNR.

#     """
#     spectrum = spectrum / spectrum.flux.mean()
    
#     p_coeff = np.polyfit(SNR[:,0]*10, SNR[:,1], 4, rcond=None, full=False, w=None, cov=False)
    
#     pol = np.poly1d(p_coeff)
    
#     spectrum = sp.Spectrum1D(
#         spectral_axis = spectrum.spectral_axis,
#         flux = spectrum.flux * pol(spectrum.spectral_axis.value)**2,
#         uncertainty = astropy.nddata.StdDevUncertainty(
#             np.sqrt(spectrum.flux * pol(spectrum.spectral_axis.value)**2)
#             )
#         )
    
#     return spectrum

# #%%
# def generate_mock_spectrum(
#     TargetParameters: para.SystemParametersComposite,
#     instrument: str = 'ESPRESSO',
#     exptime: u.Quantity = 900*u.s,
#     ) -> sp.Spectrum1D:
#     """
#     Generate mock spectrum using the PHOENIX high resolution spectra.

#     Parameters
#     ----------
#     TargetParameters : para.SystemParametersComposite
#         Target parameters as loaded by rats.para.SystemParametersComposite class
#     instrument : str, optional
#         Which instrument to consider. The default is 'ESPRESSO'.

#     Returns
#     -------
#     best_scenarion_spectrum : sp.Spectrum1D
#         Mock spectrum interpolated to instrument wavelength grid assuming the best weather conditions.
#     average_scenarion_spectrum : sp.Spectrum1D
#         Mock spectrum interpolated to instrument wavelength grid assuming the average weather conditions.
#     worst_scenarion_spectrum : sp.Spectrum1D
#         Mock spectrum interpolated to instrument wavelength grid assuming the worst weather conditions.

#     """
#     stellar_spectrum = TargetParameters.Star.stellar_model()
    
#     new_stellar_spectrum = sm.interpolate2commonframe(
#         stellar_spectrum,
#         inst.Instruments[instrument]
#         )
    
#     best_scenario, average_scenario, worst_scenario = _load_SNR_table_ESPRESSO(
#         TargetParameters,
#         instrument = instrument
#         )
    
#     best_scenario = scale_estimated_SNR_with_Vmag(best_scenario, TargetParameters)
#     average_scenario = scale_estimated_SNR_with_Vmag(average_scenario, TargetParameters)
#     worst_scenario = scale_estimated_SNR_with_Vmag(worst_scenario, TargetParameters)
    
#     # FIXME: This is ESPRESSO specific, do a branch for other instrument. 
#     # Right now, this is fixed by _scale_instrument function, but should be unified
#     best_scenario_spectrum = _scale_spectrum_with_SNR(new_stellar_spectrum, best_scenario)
#     average_scenario_spectrum = _scale_spectrum_with_SNR(new_stellar_spectrum, average_scenario)
#     worst_scenario_spectrum = _scale_spectrum_with_SNR(new_stellar_spectrum, worst_scenario)
    
#     if instrument != 'ESPRESSO' or instrument != 'ESPRESSO_4UT':
#         best_scenario_spectrum = _scale_instrument(best_scenario_spectrum, 'ESPRESSO', instrument)
#         average_scenario_spectrum = _scale_instrument(average_scenario_spectrum, 'ESPRESSO', instrument)
#         worst_scenario_spectrum = _scale_instrument(worst_scenario_spectrum, 'ESPRESSO', instrument)
    
#     if exptime != 900*u.s:
#         best_scenario_spectrum = best_scenario_spectrum.multiply(exptime/900*u.s)
#         average_scenario_spectrum = average_scenario_spectrum.multiply(exptime/900*u.s)
#         worst_scenario_spectrum = worst_scenario_spectrum.multiply(exptime/900*u.s)
    
#     best_scenario_spectrum, average_scenario_spectrum, worst_scenario_spectrum = _add_mask([best_scenario_spectrum, average_scenario_spectrum, worst_scenario_spectrum])
    
#     return best_scenario_spectrum, average_scenario_spectrum, worst_scenario_spectrum


# def _add_mask(dataset: sp.SpectrumList) -> sp.SpectrumList:
#     """
#     Adds a mask to a dataset based on its flux element.

#     Parameters
#     ----------
#     dataset : sp.SpectrumList
#         Dataset to add mask in.

#     Returns
#     -------
#     dataset : sp.SpectrumList
#         Dataset with mask argument.
#     """
#     for spectrum in dataset:
#         spectrum.mask = np.isnan(spectrum.flux)
    
#     return dataset